Added 863 files from NVIDIA/cccl sparse checkout: - c2h/ (27 files): Catch2 test helpers — generators, validators, runner - nvbench_helper/ (10 files): Benchmark harness utilities - cmake/ (29 files): CMake presets and build helpers - cudax/ (794 files): Experimental CUDA extensions - AGENTS.md: NVIDIA's official AI agent instructions for CCCL - CMakePresets.json: Standardized build configurations - cccl-version.json: Version tracking Also added CCCL_ASSET_MAP.md mapping all 4295 CCCL files to competition value and PRD items. cccl_upstream now covers 100% of competition-critical assets: - 27 tuning headers (SM80/90/100 benchmark data) - 32 dispatch headers (algorithm implementations) - 60 Thrust examples (correctness verification) - 217 CUB Catch2 tests (regression matrix) - 153 CUB benchmarks (parameter space search) - 18 CUB examples (API verification) - 27 test helpers + benchmark harness - 794 cudax experimental extensions
74 lines
1.6 KiB
Plaintext
74 lines
1.6 KiB
Plaintext
//===----------------------------------------------------------------------===//
|
|
//
|
|
// Part of CUDASTF in CUDA C++ Core Libraries,
|
|
// under the Apache License v2.0 with LLVM Exceptions.
|
|
// See https://llvm.org/LICENSE.txt for license information.
|
|
// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
|
|
// SPDX-FileCopyrightText: Copyright (c) 2022-2024 NVIDIA CORPORATION & AFFILIATES.
|
|
//
|
|
//===----------------------------------------------------------------------===//
|
|
|
|
/**
|
|
* @file
|
|
*
|
|
* @brief An AXPY kernel described using a cuda_kernel construct
|
|
*
|
|
*/
|
|
|
|
#include <cuda/experimental/stf.cuh>
|
|
|
|
using namespace cuda::experimental::stf;
|
|
|
|
__global__ void axpy(double a, slice<const double> x, slice<double> y)
|
|
{
|
|
int tid = blockIdx.x * blockDim.x + threadIdx.x;
|
|
int nthreads = gridDim.x * blockDim.x;
|
|
|
|
for (int i = tid; i < x.size(); i += nthreads)
|
|
{
|
|
y(i) += a * x(i);
|
|
}
|
|
}
|
|
|
|
double X0(int i)
|
|
{
|
|
return sin((double) i);
|
|
}
|
|
|
|
double Y0(int i)
|
|
{
|
|
return cos((double) i);
|
|
}
|
|
|
|
int main()
|
|
{
|
|
context ctx = graph_ctx();
|
|
const size_t N = 16;
|
|
double X[N], Y[N];
|
|
|
|
for (size_t i = 0; i < N; i++)
|
|
{
|
|
X[i] = X0(i);
|
|
Y[i] = Y0(i);
|
|
}
|
|
|
|
double alpha = 3.14;
|
|
|
|
auto lX = ctx.logical_data(X);
|
|
auto lY = ctx.logical_data(Y);
|
|
|
|
/* Compute Y = Y + alpha X */
|
|
ctx.cuda_kernel(lX.read(), lY.rw())->*[&](auto dX, auto dY) {
|
|
// axpy<<<16, 128, 0, ...>>>(alpha, dX, dY)
|
|
return cuda_kernel_desc{axpy, 16, 128, 0, alpha, dX, dY};
|
|
};
|
|
|
|
ctx.finalize();
|
|
|
|
for (size_t i = 0; i < N; i++)
|
|
{
|
|
assert(fabs(Y[i] - (Y0(i) + alpha * X0(i))) < 0.0001);
|
|
assert(fabs(X[i] - X0(i)) < 0.0001);
|
|
}
|
|
}
|